/*
 *
 * @(#)DataTrace.java 0.1 97/07/31
 *
 * Copyright (c) 1997 Ginger Booth.  All Rights Reserved.
 *
 * @author Ginger Booth
 * @version 0.1, August 1997
 */
package courseware.plot;

import java.awt.Color;
import java.util.Vector;

/** One trace of a plot--an object for putting data in.
 */
public class DataTrace
{
    private Color color;
    private String label;
    private double x[];
    private double y[];

    private int allocChunk = 50;
    private int npts = 0;

    /** note color and label are optional -- black and blank default */
    public DataTrace ()
    {
        color        = new Color (0,0,0);
        x            = null;
        y            = null;
    }

    public double[] getX() { return x; }
    public double getX(int i) {return x[i];}
    public double getLastX() {return x[npts-1];}
    public void setX(double x[]) { this.x = x; npts = x.length; }
    public void setX(int i, double val) { x[i] = val;}
    public double[] getY() { return y; }
    public double getY(int i) { return y[i];}
    public double getLastY() {return y[npts-1];}
    public void setY(double y[]) { this.y = y; npts = y.length; }
    public void setY(int i, double val) { y[i] = val;}
    public int getNpts() { return npts;}
    public void setNpts(int n) { npts = n;}
    public double getMaxY() {
      double max = y[0];
      for (int i=1; i<y.length; i++)
        if (y[i] > max)
          max = y[i];
      return max;
    }

    /** if you know how many, say so */
    public DataTrace (int alloc)
    {
        color = new Color (0,0,0);
        allocate(alloc);
        npts = 0;
    }

    public void allocate(int ptArraySize) {
      x = new double[ptArraySize];
      y = new double[ptArraySize];
    }
    public void add(double xx, double yy) {
        if (x == null)
          x = new double[allocChunk];
        if (y == null)
          y = new double[allocChunk];
        if (npts == x.length) {
            int curlen = x.length;
            double t[] = new double[curlen * 2];
            System.arraycopy(x, 0, t, 0, npts);
            x = t;
            t = new double[curlen * 2];
            System.arraycopy(y, 0, t, 0, npts);
            y = t;
        }
        x[npts] = xx;
        y[npts] = yy;
        npts++;
    }
    public String getLabel() { return label; }
    public void setLabel(String l) {label = l;}
    public void setColor(Color c) { color = c; }
    public Color getColor() { return color;}

    public void clear() {
        npts = 0;
        x = null;
        y = null;
    }

    public String toString() {
      String answer = "DataTrace[label=" + label ;
      if ((x==null) || (y == null))
        return answer + " x=null, y=null]";
      if ((x.length >= 0) && (y.length >= 0)) {
        answer += "pts=" + x.length + ",x0=" + x[0] + ",y0=" + y[0] + "...]";
      }
      return answer;
    }
}

