/*
 *
 * @(#)PlotMarker.java 000922
 *
 * Copyright (c) 2000 Ginger Booth.  All Rights Reserved.
 * Author:       Ginger Booth, ginger@gingerbooth.com
 * Company:      Yale IBS/CCE
 * Description:  Plot traces + markers (points)
 *
 * Created 000922, vfb, for eco.diversity labs.
 */

package courseware.plot;
import java.util.Vector;
import java.awt.*;
import java.applet.*;
import courseware.jfcdep.*;

public class PlotMarker extends PlotFn {
  private Vector markers;
  private int chunk = 10; // Vector allocation chunk

  public PlotMarker() {
    super();
    markers = new Vector(chunk, chunk);
  }
  public PlotMarker(Dimension size, String x_l, String y_l,
                double xMin, double yMin, double xMax, double yMax,
                boolean legend, Applet owner)
  {
    super(size, x_l, y_l, xMin, yMin, xMax, yMax, legend, owner);
    markers = new Vector(chunk, chunk);
  }

  public void addMarker(Marker m) {
    markers.addElement(m);
    setNeedRepaint(true);
  }
  public void clearMarkers() {
    markers = new Vector(chunk, chunk);
    setNeedRepaint(true);
  }

  synchronized protected void plot_data (Graphics g) {
    synchronized (g) {
      super.plot_data(g);
      int num = markers.size();
      Marker m;
      for (int i=0; i<num; i++) {
        m = (Marker) markers.elementAt(i);
        g.setColor(m.color);
        int cx = x_pixel(m.x);
        int cy = y_pixel(m.y);
        int size = m.pixSize;
        int half = m.pixSize / 2;
        if (insideGraph(cx, cy)) {
          switch (m.style) {
            case Marker.CIRCLE :
              g.drawOval(cx - half, cy - half, size, size);
              break;
            case Marker.SQUARE :
              g.drawRect(cx - half, cy - half, size, size);
              break;
            case Marker.FCIRCLE :
              g.drawOval(cx - half, cy - half, size, size);
              g.fillOval(cx - half, cy - half, size, size);
              break;
            default :
              g.drawRect(cx - half, cy - half, size, size);
              g.fillRect(cx - half, cy - half, size, size);
              break;
          }
        }
      }
    }
  }
}
