package courseware.util;
import java.awt.Color;

/**
 * Title:        Courseware
 * Description:
 * Copyright:    Copyright (c) 1998-2001
 * Company:      Yale Inst. Biospheric Studies
 * @author Ginger Booth
 * @version 3.0
 *
 * Some colors that look good on grey (or white...) plots.
 */

public class PalettePlots extends Palette {
  public static final int WHITE = 0;
  public static final int GREY = 1;
  public static final int BLACK = 2;
  public static final int YELLOW = 3;
  public static final int OCHRE = 4;
  public static final int ORANGE = 5;
  public static final int MELON = 6;
  public static final int RED = 7;
  public static final int HOTPINK = 8;
  public static final int PURPLE = 9;
  public static final int LAVENDER = 10;
  public static final int GREYBLUE = 11;
  public static final int LTBLUE = 12;
  public static final int BLUE = 13;
  public static final int BLUEGREEN = 14;
  public static final int OLIVE = 15;
  public static final int GREEN = 16;

  public static final int[] best6 =
    {BLACK, OCHRE, RED, HOTPINK, PURPLE, BLUE};
  public static final int[] tier2 =
    {GREY, MELON, LAVENDER, GREYBLUE, LTBLUE, BLUEGREEN, GREEN};
  public static final int[] tier3 =
    {WHITE, YELLOW, ORANGE, OLIVE};
  public static final int[] best13 =
    {BLACK, GREY, OCHRE, MELON, RED, HOTPINK, PURPLE, LAVENDER,
     GREYBLUE, LTBLUE, BLUE, BLUEGREEN, GREEN};
  public static final int[] best11Color =
    {OCHRE, MELON, RED, HOTPINK, PURPLE, LAVENDER,
     GREYBLUE, LTBLUE, BLUE, BLUEGREEN, GREEN};
  public static final int[] all =
    {WHITE, GREY, BLACK, YELLOW, OCHRE, ORANGE, MELON, RED,
    HOTPINK, PURPLE, LAVENDER, GREYBLUE, LTBLUE, BLUE, BLUEGREEN,
    OLIVE, GREEN};
  public static final int[] allColor =
    {YELLOW, OCHRE, ORANGE, MELON, RED,
    HOTPINK, PURPLE, LAVENDER, GREYBLUE, LTBLUE, BLUE, BLUEGREEN,
    OLIVE, GREEN};

  /** gives the closest palette we've got to the number requested... */
  public PalettePlots(int num) {
    super();
    if (num <= 6)
      set(best6);
    else if (num <= 11)
      set(best11Color);
    else if (num <= 13)
      set(best13);
    else if (num <= 14)
      set(allColor);
    else
      set(all);
  }

  /** Either send one of the prebuilt selections above, or an integer
   *  array of named colors above. Any unknown int's get neon green.
   */
  public PalettePlots(int[] selection) {
    super();
    set(selection);
  }

  public void set(int[] selection) {
    for (int i=0; i<selection.length; i++) {
        addColor(makeColorObj(selection[i]));
    }
  }
  static public ColorObj makeColorObj(int code) {
      Color c;
      String name;
      switch(code) {
        case WHITE : c = new Color(255,255,255); name="white"; break;
        case GREY  : c = new Color(153,153,153); name="grey"; break;
        case BLACK : c = new Color(0,0,0); name="black"; break;
        case YELLOW: c = new Color(255,255,51); name="yellow"; break;
        case OCHRE:  c = new Color(204,153,51); name="ochre"; break;
        case ORANGE: c = new Color(255,204,51); name="orange"; break;
        case MELON:  c = new Color(255,153,153); name="melon"; break;
        case RED  :  c = new Color(255,51,51); name="red"; break;
        case HOTPINK:c = new Color(255,51,153); name="hotpink"; break;
        case PURPLE: c = new Color(204,51,255); name="purple"; break;
        case LAVENDER:c = new Color(204,153,204); name="lavender"; break;
        case GREYBLUE:c = new Color(153,153,204); name="greyblue"; break;
        case LTBLUE: c = new Color(51,204,255); name="ltblue"; break;
        case BLUE:   c = new Color(51,51,255); name="blue"; break;
        case BLUEGREEN:c = new Color(51,204,204); name="bluegreen"; break;
        case OLIVE:  c = new Color(153,204,102); name="olive"; break;
        case GREEN:  c = new Color(51,204,51); name="green"; break;
        default: c = new Color(0,0,255); name="default";
      }
      return new ColorObj(c, name);
  }
  static public Color makeColor(int code) {
      ColorObj co = makeColorObj(code);
      return co.color;
  }

}

