//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator


package gecko;

import courseware.util.*;
import courseware.plot.*;
import courseware.jfcdep.*;
import java.awt.*;
import java.util.*;

import stats.*;

public class GeckoCourseWare extends CourseWareAdapter implements GeckoDriver {
  private Simulator simulator;
  private ParamList currentParams;
  private UI ui;
  private String defaultBGLoc            = "images/dark13.gif";
  private String defaultPlainButtonLoc   = "images/copper54x20.gif";
  private String defaultPressedButtonLoc = "images/darkcopper54x20.gif";
  private boolean bacsimFlag = false;

  public GeckoCourseWare() {
    super();
    Globals.setGeckoDriver(this);
    if (ui != null)
      ui.finalize();
    ui = new AppletGUI(new Globals());
    Globals.setUI( ui );
    Globals.setStats( new Stats( ui ) );
  }

  public void setBacsim(boolean b) {bacsimFlag = b;}
  public boolean isBacsim() {return bacsimFlag;}

  /** You CANNOT restart inside applet. Period. */
  public boolean isRestart() {return false;}

  public void setBackgroundImage(String s) {
    CWGlobals.setBackgroundImage(s, cwaOwner());
  }
  public void setBackgroundImageOff() {
    CWGlobals.bgPainter = null;
  }

  public void setButtonImages(String plain, String pressed) {
    CWGlobals.setButtonImages(0, plain, pressed, cwaOwner());
  }

  public void init() {
    super.init();
    setBackgroundImage(defaultBGLoc);
    setButtonImages(defaultPlainButtonLoc, defaultPressedButtonLoc);
    simulator = new Simulator();
    Globals.setSimulator(simulator);
    simulator.setDriver(this);
    simulator.makeWorld();
    makeControlButtons();
  }

  public void bindParams(ParamList p) {
    p.bindAllObjects(this.getClass(), this);
    // GeckoDriver is the common i/f for applet and application modes.
    p.bindAllObjects(GeckoDriver.class, this);
    p.bindAllObjects(Globals.getWorldDisplay().getClass(), Globals.getWorldDisplay());
    p.bindAllObjects(simulator.getClass(), simulator);
    currentParams = p;
  }
  public ParamList getParams() {
    return currentParams;
  }

  public void clear() {
    super.clear();
    simulator.clear();
  }

  public void makeControlButtons() {
    GeckoDriverCommon.makeControlButtons(this, simulator);
  }

  /** Set background of quaddisplay and plotters.*/
  public void setBackground(Color c) {
    super.setBackground(c);
    Globals.getWorldDisplay().setBackground(c);
    Plotter.setDefBackgroundColor(c);
  }
  public void setButtonBackground(Color c) {
    MatchedButton.background = c;
  }
  public void setButtonForeground(Color c) {
    MatchedButton.foreground = c;
  }

  /** This does nothing. But permits same protocol to be used for Application
   *  and Applet. The Applet version doesn't write stats to file.*/
  public void setOutputFileStem(String s){  }
  /** Also does nothing--applet mode. */
  public void setNumRuns(int i){  }
  public int getNumRun() {return 0;}

  /** Set foreground of quaddisplay and plotters.*/
  public void setForeground(Color c) {
    GeckoDriverCommon.setForeground(this, c);
  }

  public void setGridColor(Color c) {
    GeckoDriverCommon.setGridColor(c);
  }

  public void addAllSpeciesTo(String paramListName) {
    GeckoDriverCommon.addAllSpeciesTo(this, paramListName);
  }
  public void addStatsTo(String paramListName, boolean includeSites) {
    GeckoDriverCommon.addStatsTo(this, paramListName, includeSites);
  }
  public void addStatsTo(String paramListName) {
    GeckoDriverCommon.addStatsTo(this, paramListName, true);
  }

  public void update (Graphics g) { paintComponents(g); }

  public void start() {
    Globals.setCurrentTime(0);
    getOwner().doLayout();
    doLayout();

    // was to 010111, vfb:

    synchronized (simulator) {
      simulator.postParams(); // does post-params init on world
      simulator.start();
    }

  }

  public void finalize() {
    if (ui != null)
      ui.finalize();
    ui = null;
    super.finalize();
  }

  public void stop() {
    /* 001102, changed ala DaisyMgr to make scene change work under Swing */
    if (simulator.getThread().isAlive()) {
      simulator.getThread().interrupt();
      simulator.getThread().stop();
    }
    simulator.clear();
    simulator.close();
    if (ui != null)
      ui.clear();
    ui = null;
    MemCleaner.nap();
  }

}
