//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/** Glue one bucket of BiomassTransfers onto plot/file output. */

package gecko.statglue;
import stats.*;
import gecko.*;

public class BiomassBucketBroker extends StatSourceBroker {
  private double scale;
  private int bucket;
  private boolean weightFlag;

  /** If the biomass stat is a rate, it is not weighted by the resource scale
   *  factor. If the optional setWeightFlag is set, getWeight returns biomass.
   *  That's used when the broker is added to an AvgTrace.
   */
  public BiomassBucketBroker(BiomassTransfers source, int bucket,
    int resourceId, boolean rateFlag)
  {
    statsource = (StatSource) source;
    this.bucket = bucket;
    if (rateFlag)
      scale = 1.0;
    else
      scale = Globals.getResources().getBioScale(resourceId);
    weightFlag = false;
  }
  public BiomassBucketBroker(BiomassBucketBroker toCopy)
  {
    statsource = toCopy.statsource;
    bucket = toCopy.bucket;
    weightFlag = toCopy.weightFlag;
    scale = toCopy.scale;
  }
  public double getValue() {
    return ((BiomassTransfers) statsource).getBucket(bucket);
  }
  public double getWeight() {
    if (weightFlag)
      return ((BiomassTransfers)statsource).getBucket(BiomassTransfers.BIOMASS);
    else
      return scale;
  }
  public void setWeightFlag(boolean b) {weightFlag = b;}
}
