//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 991027, vfb, original Java
 * Lifecycle stages by size-as-tree-height.
 */
package gecko.woods;
import gecko.*;
import gecko.statglue.*;
import gecko.zoo.*;
import courseware.util.*;
import courseware.plot.*;
import courseware.jfcdep.*;

public class HeightStageObserver implements StageObserver, GetPropertyNamesable {
  // additional advertised Species parameters for this species
  private int numStages;  // number of sizes to divide agents into
  private double stageMinHeight;
  private double stageMaxHeight;

  // criteria and labels
  private String[] stageLabels = null; // generated first time asked for
  private double[] stageHeights = null; // generated first time asked for

  // rigmarole to get my parameters registered on menus
  private static String[] props = { "numStages", "stageMinHeight", "stageMaxHeight" };
  public String[] getPropertyNames() { return props;}

  public HeightStageObserver() {
  }
  // get/set public properties

  public void setNumStages(int v) {numStages = v; }

  public double getStageMinHeight(){return stageMinHeight;}

  public void setStageMinHeight(double v) {stageMinHeight = v;}

  public double getStageMaxHeight(){return stageMaxHeight;}

  public void setStageMaxHeight(double v) {stageMaxHeight = v;}


  /** Implement interface StageObservable.*/

  public int getNumStages() {
    if (stageHeights==null)
      makeStages();
    return numStages;
  }

  /** stage id of this critter (size0, ..., sizeStageN-1 */
  public int getStage(StageObservable agent) {
    if (!(agent instanceof QuadCircleSpecies))
      return 0;
    QuadCircleSpecies critter = (QuadCircleSpecies)agent;
    for (int i=0; i<numStages-1; i++) {
      if (critter.getZ() < stageHeights[i])
        return i;
    }
    return numStages - 1;
  }

  /** stage plot label for a stage id (like "h > 3.367") */
  public String getStageName(int i) {
    if (stageLabels == null) {
      return "All";
    }
    return stageLabels[i];
  }

  /** First time I'm asked, need to build criteria and label list. */
  private void makeStages() {
    if (numStages == 0) {
      stageHeights = new double[] { Double.POSITIVE_INFINITY };
      stageLabels = new String[] { "All" };
    } else {
      stageHeights = new double[numStages];

      double heightSpan = stageMaxHeight - stageMinHeight;
      double heightDelta = heightSpan / numStages;
      double heightNext = stageMinHeight;

      for (int i=0; i<numStages; i++) {
        heightNext += heightDelta;
        stageHeights[i] = heightNext;
      }

      stageLabels = new String[numStages];
      for (int j=0; j<numStages - 1; j++) {
        stageLabels[j] = "h <= " + TruncVal.truncVal(stageHeights[j], 4);
      }
      stageLabels[numStages - 1] = "h > " + TruncVal.truncVal(stageHeights[numStages - 2], 4);
    }
  }
}


