//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 991206, vfb, original Java
 * ReaperWind grants extra random damage every year to any tree that
 * has less than 3/4 of its neighborhood (radiusFactor x its radius) covered by
 * tree crowns. The damage runs from min to max -by year-. For a given
 * year, all exposed trees get that damage times their personal exposure.
 * Trees with an exposure of less than 1/2 their neighborhood get 0 damage.
 * There's an option here to factor in crown closedness, idea being that
 * denser crowns tend to protect themselves better. In this case, for example,
 * Hemlock get .06 * (min..maxdamage).
 */

package gecko.woods;
import gecko.*;
import gecko.zoo.QuadCircleSpecies;
import courseware.jfcdep.TruncVal;

public class ReaperWind implements Reapable, Steppable {
  // defining
  double minDamage, maxDamage;
  boolean useCrownFactor;
  double radiusFactor = 2; // use 2x radius for exposure checks
  double exposureThreshold = .5; // can't use less than .25 (tree itself takes up .25)

  // working
  private double thisYearDamage;
  private McrCircle hood = new McrCircle(); // always need one, don't realloc
  private McrCircle ncircle = new McrCircle(); // always need one, don't realloc
  private McrCircle cross = new McrCircle(); // always need one, don't realloc

  public ReaperWind() {
    minDamage=maxDamage=0;
    useCrownFactor = false;
    thisYearDamage = 0;
    radiusFactor = 2;
  }
  private int damageCount;
  private double damageAmount;
  private double avgExposure;
  private int allcount;
  public void step() {
    thisYearDamage = minDamage + Globals.getRandomDouble() * (maxDamage-minDamage);
    if ((thisYearDamage > 0) && (Globals.getCurrentTime() % Globals.getStatCycle()) == 0) {
      Globals.log("Wind damaged " + damageCount +
        " trees, avg damage " + TruncVal.stringVal( damageAmount/damageCount ,5) +
        ", avg exposure " + TruncVal.stringVal( avgExposure/allcount, 4) +
        " last year; for time " + Globals.getCurrentTime() +
        " inflict " + TruncVal.stringVal(thisYearDamage, 5));
      }
    damageCount = 0;
    damageAmount = 0;
    avgExposure = 0;
    allcount = 0;
  }
  public double findExposure(ReapableTree aTree) {
    // really shouldn't use a diffusive lightbox, but breakboxing takes 5x as
    // long or longer, and the answers aren't all that different.
    // Woods.setDiffusiveLightBox(false);
    QuadCircleSpecies tree = (QuadCircleSpecies) aTree;
    Woods.setDiffusiveLightBox(true);
    hood.circleToSquare(tree.x, tree.y, tree.radius * radiusFactor);
    // paint the 'hood light
    Lightbox lb = new Lightbox(hood, 1.0);
    Quad plantRoot = Globals.getPlantRoot();
    Quad quad = plantRoot.pickWin(hood, plantRoot, true);
    Tree neighbor;
    while (quad != null) {
      // note that we -want- to factor in this tree as a debit on the 'hood.
      if (quad instanceof Tree) {
        neighbor = (Tree)quad;
        ncircle.circleToSquare(neighbor);
        if (hood.intersectMcr(ncircle, cross) != Mcr.MCR_DISJOINT) {
          // color each tree area dark....
          lb.factorIn(cross, 0);
        }
      }
      quad = quad.pickWin(hood, plantRoot, true);
    }
    // and answer (exposure) should be remaining light density
    return lb.getLightDensity();
  }
  public void reap(ReapableTree tree) {
    if (thisYearDamage == 0)
      return;
    double exposure = findExposure(tree);
    avgExposure += exposure;
    allcount++;
    if (exposure <= exposureThreshold) return;
    double damage = exposure * thisYearDamage;
    if (useCrownFactor)
      damage *= tree.getLightCoeff();
    tree.crownDamage(damage);
    damageAmount+= damage;
    damageCount++;
  }
  public void setMinDamage(double d) {minDamage = d;}
  public double getMinDamage() {return minDamage;}
  public void setMaxDamage(double d) {maxDamage = d;}
  public double getMaxDamage() {return maxDamage;}
  public void setRadiusFactor(double d) {radiusFactor = d;}
  public double getRadiusFactor() {return radiusFactor;}
  public boolean isCrownFactor(){return useCrownFactor;}
  public void setCrownFactor(boolean v) {useCrownFactor = v;}
}
