//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 2000, All rights reserved
//Author:       Ginger Booth, ginger@gingerbooth.com, gingerbooth@cyber-wizard.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 000727, original: invariant species parameters for Tree2. All fields are
 * communicated via Tree, not here. Package-scope publicity. Delegate record of
 * Tree2.
 *
 * Need this to add partition schedules to Tree2.
 */


package gecko.woods;
import gecko.Globals;

public class Tree2Params extends TreeParams {
  // The partition (% growth going to this part of Tree2) for leaf & stem.
  double leafPart;
  double stemPart;
  double leafTurnover; // % of leaves to be replaced per year
  // note: I do not know what crownDensity should be. So calculate it.
  private double crownDensity; // relative to wood, on cone of crownDepth, crownRadius

  /** Just a utility routine. */
  static public double coneVolume(double r, double h) {
    return (Math.PI / 3.0) * r * r * h;
  }
  /** convert back, return -radius for -volume */
  static public double coneRadius(double vol, double h) {
    double inside = (vol * 3.0) / (h * Math.PI);
    double ans = Math.sqrt( Math.abs( (vol * 3.0) / (h * Math.PI) ) );
    if (inside < 0)
      ans = - ans;
    return ans;
  }

  /** freshWood is the living part of the stem. Since how much of the crown
   *  is included in "stem" matters in several places, localize the scheme
   *  du jour. Density 1.0 this-wood-density (whatever that is).
   * NOTE: deltaBA is in cm since stemR is kept in cm. Returns m^3.
   */
  static public double getFreshWood(double height, double crownDepth, double deltaBA) {
    // hollow cyclinder, delta-basal-area * height. The height used is to halfway
    // up the crown.
    double freshWood = (height - crownDepth/2.0) * deltaBA;
    freshWood /= 10000; // because ba is in cm and height is in m.
    return freshWood;
  }

  /** Noplace to reliably put a check for whether the crownDensity derived
   *  value is current. So, to be safe, if this time is less than last time,
   *  or this time is zero, always calculate it.
   */
  private long lastCrownTime = Long.MAX_VALUE;
  public double getCrownDensity(){
    long now = Globals.getCurrentTime();
    if ( (now < lastCrownTime) || (now == 0) ) {
      // Figure out what density should be. At maxHeight, maxCrown, leaf volume
      // loss and gain are equal, and freshWood is constant (should be using
      // growRingMax anywhere near maxHeight.)
      // Then density = leafBudgetMass / leafVolumeLoss (= gain).
      double calcHeight = 0.95 * maxHeight; // asymptotic....
      double freshWood = getFreshWood(calcHeight, calcHeight * crownDepthToHeight, growRingMax);
      double growthBudget = freshWood / stemPart;
      double leafMassBudget = growthBudget * leafPart;
      double leafVolume = coneVolume(maxCrownRadius, calcHeight * crownDepthToHeight);
      double leafVolLossGain = leafVolume * leafTurnover;
      crownDensity = leafMassBudget / leafVolLossGain;
    }
    lastCrownTime = now;
    return crownDensity;
  }



}