//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990929, vfb, original Java
 * Most basic thing meeting the Speciesable requirements and abiding
 * on the Quad tree. Split off from Critter. Critter adds metabolism and
 * presumptions of step behavior sequence and reproduction and movement, etc.
 * Alive just meets the minimum bookkeeping requirements. Z support isn't
 * strictly necessary, but it's a small price to pay to avoid putting it higher
 * in the hierarchy.
 *
 * Note that the default QuadCircleSpecies.step does nothing. In other words,
 * instances of this gizmo are completely passive, but show up. As circles.
 */

package gecko.zoo;
import gecko.*;
import java.awt.Color;

public class QuadCircleSpecies extends Quad implements Speciesable, GetPropertyNamesable {
  private Species species; 	    // name, birthing spiel, stats, etc.

  /** This property list is used to generate paramLists for popup editing.
   *  Note initArea is left off for now, as don't have the routines to set
   *  it nicely from a popup. x, y similarly irrelevant for popup setting,
   *  since one is setting the progenitor before strewing in the world.
   *  Radius is relevant, but non-obvious:  it's the radius a child is born
   *  into the world at.
   */
  protected static String[] props = { "radius" };

  // working settings
  // color, xy, and radius are public so that QuadDisplay can go faster
  public int color = QuadDisplay.setColor("Alive", Color.green);
  public double x, y, z;		  // center - z usually ignored
  public double radius;	      // current size
  protected long birthday;	    // getCurrentTime() at birth
  protected int biomassId = 1;// convenience

  public QuadCircleSpecies() {
    super();
    birthday = Globals.getCurrentTime();
    setType(Quad.QUAD_CRITTER); // for QuadDisplay purposes
  }

  public void setColor(String name, Color c) {
    color = QuadDisplay.setColor(name, c);
  }
  public void setColor(String name, int r, int g, int b) {
    color = QuadDisplay.setColor( name, new Color(r, g, b) );
  }

  public void setSpecies(Species s) {species = s;}
  public Species getSpecies() {return species;}
  public void setColor(int i) {color = i;}
  public int getColor() {return color;}
  /** used to build paramLists for popups. */
  public String[] getPropertyNames() {return props;}
  /** setX, Y, radius is optional for Circleable, not for Living. */
  public void setX(double x) {this.x = x; setMcr();}
  public void setY(double y) {this.y = y; setMcr();}
  public void setZ(double z) {this.z = z; setMcr();}
  public void setRadius(double r) {radius = r; setMcr();}
  public void setBirthday(long i) {birthday = i;}

  public double getX() {return x;}
  public double getY() {return y;}
  public double getZ() {return z;}
  public double getRadius() {return radius;}
  public long getAge() {return Globals.getCurrentTime() - birthday;}
  public long getBirthday() {return birthday;}
  /** I pick plantRoot, since I'm as passive as they come...*/
  public boolean putOnPlantRoot() {return true;}
  public void setFrom(QuadCircleSpecies other) {
    super.setFrom(other);
    x = other.x;
    y = other.y;
    z = other.z;
    radius = other.radius;
    color = other.color;
    species = other.species;
    // birthday is not heritable
  }

  public Object clone() {
    return makeKid();
  }

  public Speciesable makeKid() {
    QuadCircleSpecies kid = new QuadCircleSpecies();
    kid.setFrom(this);
    // not done, because this species isn't StageObservable
    // getSpecies().notifyBirthToMom(this);
    return kid;
  }
  /** used to put biomass on plots. */
  public int getBiomassResourceId() {return biomassId;}

  public void setMcr() {
    lx = x - radius;
    ly = y - radius;
    ux = x + radius;
    uy = y + radius;
    growMcr(); // grow sorting quads above me as needed
  }

  public String toString() {
    return "QuadCircle[x=" + x + ",y=" + y +",r=" + radius + ",color=" + color;
  }
}
