//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990811, vfb, original Java
 * Translated from the Swarm Objective-C Gecko source, version 0.51
 * With structural changes.
 */
/** A predator to eat ScaredyGrazer. Main difference from normal Carnie
 *  is that this one fails to go after prey that are "hidden" (response to
 *  Mcr exposure() -> isNullMcr().)
 *  Also, move() locks this carnie to the landRoot mcr, resulting in somewhat
 *  fewer deaths-by-stupidity. The basic Carnie wasn't taught that trick simply
 *  because those scenarios were already published.
 *
 *  This Carnie can be used with any Grazer, not just ScaredyGrazer.
 */

package gecko.zoo;
import java.util.Vector;
import gecko.*;
import courseware.jfcdep.TruncVal;

public class ScaredyGrazerCarnie extends Carnie {
  protected boolean toothlessFlag = false; // can eat unless told otherwise
  protected static String[] myprops = { "toothless" };
  private static String[] props;
  static {
    String[] firsthalf = Carnie.props;
    props = new String[firsthalf.length + myprops.length];
    for (int i=0; i<myprops.length; i++) {
      props[i] = myprops[i];
    }
    for (int i=0; i<firsthalf.length; i++) {
      props[myprops.length+i] = firsthalf[i];
    }
  }
  public String[] getPropertyNames() { return props;}

  public ScaredyGrazerCarnie() {super();}

  public boolean isToothless() { return toothlessFlag;}
  public void setToothless(boolean flag) { toothlessFlag = flag;}

  public void setFrom(ScaredyGrazerCarnie other) {
    super.setFrom(other);
    this.toothlessFlag = other.toothlessFlag;
  }

  public Speciesable makeKid() {
    ScaredyGrazerCarnie kid = new ScaredyGrazerCarnie();
    kid.setFrom(this);
    getSpecies().notifyBirthToMom(this);
    return kid;
  }
  public Object clone() {
    return makeKid();
  }

  public boolean attacks(Critter other) {

    if (other instanceof Exposureable) {

      if ( ((Exposureable)other).exposure().isNullMcr() ) {

        return false;

      }

    }

    return super.attacks(other);

  }


  /** doEat = if toothless, don't eat, just go scare it. Normally, a spider would

   *  also go to an after-lunch siesta here. But if the maimed field spider
   *  is still hungry (and it is), it wouldn't do that... Hence, due to the
   *  field conditions of the experiment, ToothlessCarnies hunt more
   *  tenaciously than normal, because they spend no time digesting.
   */
  protected void doEat(Critter gopper) {
    if (toothlessFlag) {
      direction.setDirection(gopper.getX() - this.getX(), gopper.getY() - this.getY());
    } else
      super.doEat(gopper);
  }


  /** Don't leave the landRoot, you'll fall asleep and starve out there... */

  public void move() {

    // we had a plan...

    super.move();

    // inside landscape?

    Mcr landMcr = (Mcr) Globals.getLandRoot(); // root=quad=mcr

    if (landMcr.intersectMcr(this, null) == MCR_CONTAINS)

      return;


    // no good--snap into landscape

    double wasx = getX();

    double wasy = getY();

    this.snapInto(landMcr);

    setXY(this.getCenterX(), this.getCenterY());

    // now fix direction so we don't slide along the walls...

    // the third argument is a weight. 

    addDirection(getX() - wasx, getY() - wasy, 2.0 * getRadius());

  }

}














