//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990329, vfb, original Java
 * Translated from the Swarm Objective-C Gecko source, version 0.51
 * With structural changes.
 */
/** This was the basic Critter in Gecko--a plant with min/kid/breed radii.
 *  By default, onSeason, SizedPlant and all its derivatives simply die. I.e.,
 *  this is an annual.
 */


package gecko.zoo;
import gecko.*;

public class SizedPlant extends PassiveCritter {
  protected static double defaultMinSize = 3.0;
  protected static double defaultKidSize = 2.95;
  protected  static double defaultBreedSize = 11.13;
  protected  static double defaultConversionRate = .1;
  protected  static double defaultTaxRate = 0.0480;
  protected  static double defaultTaxExponent = 0.75;

  protected double minsize;
  protected double kidsize;
  protected double breedsize;

  /** This property list is used to generate paramLists for popup editing.
   *  Note initArea is left off for now, as don't have the routines to set
   *  it nicely from a popup.
   */
  private static String[] myprops = { "minSize", "kidSize", "breedSize" };
  protected static String[] props;
  static {
    String[] firsthalf = Critter.props;
    props = new String[firsthalf.length + myprops.length];
    for (int i=0; i<myprops.length; i++) {
      props[i] = myprops[i];
    }
    for (int i=0; i<firsthalf.length; i++) {
      props[myprops.length+i] = firsthalf[i];
    }
  }
  public boolean stageObservableCheck() {return true;}

  public SizedPlant() {
    super();
    init();
  }

  public void init() {
    super.init();
    getMetabolism().setConversion(GrassResources.RESOURCE_SUNWATER, GrassResources.RESOURCE_CARBOS,
      defaultConversionRate);
    getMetabolism().setTax(GrassResources.RESOURCE_CARBOS, defaultTaxRate, defaultTaxExponent);
    breedsize = defaultBreedSize;
    kidsize = defaultKidSize;
    minsize = defaultMinSize;
    setScaleRadius(1.0);
    setScaleVolume(1.0);  // no back-compatibility default
  }

  public void setFrom(SizedPlant other) {
    super.setFrom(other);
    breedsize = other.breedsize;
    minsize = other.minsize;
    kidsize = other.kidsize;
  }

  public Object clone() {
    return makeKid();
  }

  public Speciesable makeKid() {
    SizedPlant kid = new SizedPlant();
    kid.setFrom(this);
    setBirthday(Globals.getCurrentTime());
    getSpecies().notifyBirthToMom(this);
    return (Speciesable) kid;
  }
  public Move getDesiredMove() {
    double[] unit = direction.getDirection();
    double delta = 2.0 * (getRadius() - getLastRadius());
    return new Move(delta, unit[0], unit[1]);
  }

  public double getKidDistance() {
    return (getRadius()+kidsize) * Math.exp(Globals.getRandomSource().nextDouble());
  }

  public String[] getPropertyNames() { return props;}
  public boolean isStarving() {return getRadius() < minsize; }
  public boolean wantsToReproduce() {return getRadius() >= breedsize;}
  public void setBreedSize(double d) {breedsize=d;}
  public double getBreedSize() { return breedsize;}
  public void setMinSize(double d) {minsize=d;}
  public double getMinSize() { return minsize;}
  public void setKidSize(double d) {kidsize=d;}
  public double getKidSize() { return kidsize;}
  public ResourceList getKidShare() {
    double kidbody = Globals.radius2volume(kidsize,
        getScaleRadius(), getScaleVolume());
    ResourceList list = new ResourceList();
    list.list[getMetabolism().getIam()] = kidbody;
    return list;
  }

  public boolean onSeason() { die(); return true;}

  public String toString() {
    return "SizedPlant[minsize=" + minsize + ",breedsize=" + breedsize

           + ",kidsize=" + kidsize

           + ",conversion rate=" + getConversion()

           + ",<superskipped>]";

  }


}
