//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990329, vfb, original Java
 * Living things need to provide these abilities, to be useable as Species.
 * Speciesable is not enough to abide on the Quad tree, though.  For that,
 * an agent must be a Quad. In addition, to be feedable by a FeedQuad,
 * an agent must be a Critter with a Metabolism.
 *
 * There are more interfaces/Gecko abilities than provided by the basic
 * Speciesable. The most rudimentary Speciesable is QuadCircleSpecies.
 * The equivalent of SwarmGecko's Critter is SizedPlant. SwarmGecko took
 * a monolithic approach to agents, which had the advantage of being simpler.
 * This time around I'm trying the more OO floppy approach, using interfaces
 * to specify which behaviors different agent types provide. (It would also
 * be possible to use Method calls to find out if an object supports various
 * behaviors, but in practice, it's 20x slower than using interfaces. So in
 * general, I'm opting for interfaces.)
 */

package gecko.zoo;

import java.awt.Color;
import gecko.*;

public interface Speciesable extends Steppable, Paintable, Circleable {
  public void setSpecies(Species s);
  public Species getSpecies();
  public void setColor(int i);
  public int getColor();
  /** setX, Y, radius is optional for Circleable, not for Living. */
  public void setX(double x);
  public void setY(double y);
  public void setRadius(double r);
  public void setBirthday(long i);
  /** If this is a quad, which root do you want it hung on, plant or mobile?
   *  If not a quad, the answer doesn't matter, because Species won't hang it
   *  on the quad tree. */
  public boolean putOnPlantRoot(); // else mobileRoot
  public Speciesable makeKid(); /** of the proper class... Clone problem. */
  public Object clone();   /** similar...*/
  /** used to put biomass on plots. Always returning 1 will work, if you've
   *  only got one resource to track. */
  public int getBiomassResourceId();
}
