//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990809, vfb, original Java
 * Translated from the Swarm Objective-C Gecko source, version 0.51
 * With structural changes.
 */
/** A TLG becomes an adult in a certain number of timesteps. At that time,
 *  its reproductive radius is set to match. Could be fat and fertile. Could
 *  be stunted, in which case childbearing may kill the parent, though breedsize
 *  is never set lower than twice the volume of a child. In which case, a TLG may be
 *  "adult" for a long time without growing big enough to have a kid. If it
 *  ever does grow big enough, the result is a newborn and a newborn-sized oldster.
 *
 *  Note that in SwarmGecko, maxAge was in the TLG. Gecko2, maxAge in Critter.
 */

package gecko.zoo;
import gecko.*;
import gecko.statglue.*;

public class TimeLimitGrazer extends Grazer implements TimeLimited {
  private long adultTime; // timesteps birth -> reproductive size, breedradius set then
  private static String[] myprops = { "adultTime" };
  protected static String[] props;
  static {
    String[] firsthalf = Grazer.props;
    props = new String[firsthalf.length + myprops.length];
    for (int i=0; i<myprops.length; i++) {
      props[i] = myprops[i];
    }
    for (int i=0; i<firsthalf.length; i++) {
      props[myprops.length+i] = firsthalf[i];
    }
  }
  public String[] getPropertyNames() { return props;}

  private StageObserver myStageObserver = new TLStageObserver();

  private double infinitelyFat = Double.POSITIVE_INFINITY;
  public TimeLimitGrazer() {
    super();
  }
  public void setAdultTime(int timesteps) { adultTime=timesteps;}
  public long getAdultTime() { return adultTime; }
  public boolean isAdult() { return getAge() > adultTime; }

  public void setFrom(TimeLimitGrazer other) {
    super.setFrom(other);
    adultTime   = other.adultTime;
    setBreedSize(infinitelyFat); // not heritable
  }
  private static StageObserver stageObserver = new DefaultStageObserver();
  public StageObserver getStageObserver(){
    return stageObserver;
  }

  public Object clone() {
    return makeKid();
  }

  public Speciesable makeKid() {
    TimeLimitGrazer kid = new TimeLimitGrazer();
    kid.setFrom(this);
    getSpecies().notifyBirthToMom(this);
    return kid;
  }

  public boolean wantsToReproduce() {
    long timeNow = Globals.getCurrentTime();

    if (getAge() < adultTime)
      return false;

    else if (getAge() == adultTime) {
      double kidbody = getKidShare().list[getMetabolism().getIam()];
      double minbreed = Globals.volume2radius(kidbody*2.0,
            getScaleRadius(), getScaleVolume());
      if (minbreed > radius) {
        setBreedSize(minbreed);
        return false;
      } else {
        setBreedSize(radius);
        return true;
      }
    } else
      return getRadius() >= breedsize;
  }

  public String toString() {
    return "TimeLimitGrazer[adultTime=" + adultTime +
           ",super=" + super.toString() + "]";
  }

}