
//Title:        Indeco Steel Simulator
//Version:
//Copyright:    Copyright (c) 1999
//Author:       Joseph Guse & Ginger Booth
//Company:      Yale FES/CCE
//Description:  Individual-based economic simulator
//of the steel industry, ore->consumer,
//with transportation.

/** BatchUI outputs log to fileStem_log.txt AND System.err. Also prints all
 *  stats to files. Needs Timer-implementor due to time on each line of stat
 *  output files.
 */

/** Adopted from Indeco for Gecko-in-CourseWare, 990708, vfb. */

package stats;


import java.io.*;
import java.util.*;

public class BatchUI extends UI {
  private String fileStem;
  private Vector metaSinks; // of MetaSink (MetaPlot/dumpers record)
  private Timer timer;

  public BatchUI(String fileStem, String protocolFile, Timer timer) {
    super(protocolFile);
    this.fileStem = fileStem;
    this.timer = timer;
    metaSinks = new Vector(1,1);
    setLog(fileStem + "_log.txt");
    setErrorLog(System.err);
  }

  /** Hooks statistic "plot" to new output file. */
  public void makeStatDumpers(MetaPlot metaplot) {
    MetaSink ms = getMetaSink(metaplot);
    if (ms != null) {
      return; // don't do it twice...
    }

    ms = new MetaSink(metaplot);
    metaSinks.addElement(ms);
    if (fileStem != null) {
      String filename = fileStem + "_" + metaplot.getNameForFile() + ".txt";
      PrintWriter newOutput = openFile(filename);
      FileStatDumper file = new FileStatDumper(newOutput, timer);
      metaplot.showOn(file);
      metaplot.setMayTruncateTraces(true);
      ms.addStatDumper(file);
    }
  }

  /** Metaplot's been restarted, please refresh the dumpers. */
  public void refreshStatDumpers() {
    for (int i=0;i<metaSinks.size(); i++)
      ((MetaSink)metaSinks.elementAt(i)).refresh();
  }

  protected MetaSink getMetaSink(MetaPlot metaPlot) {
    for (int i=0; i<metaSinks.size(); i++) {
      MetaSink ms = (MetaSink) metaSinks.elementAt(i);
      if (ms.getMetaPlot() == metaPlot)
        return ms;
    }
    return null;
  }

  protected Vector getMetaSinks() { return metaSinks;}
  protected Timer getTimer() { return timer;}
  public String getFileStem() { return fileStem;}
  public void setFileStem(String s) { fileStem = s; }

  public Vector getStatDumpers(MetaPlot metaPlot) {
    MetaSink ms = getMetaSink(metaPlot);
    if (ms == null)
      return null;
    else
      return ms.getStatDumpers();
  }

  /** Step all the statDumpers. NOT the sources--that's StatMinder's job. */
  public void step() {
    MetaSink ms;
    StatDumper dumper;
    for (int i=0; i<metaSinks.size(); i++) {
      ms = (MetaSink)metaSinks.elementAt(i);
      Vector dumpers = ms.getStatDumpers();
      for (int j=0; j<dumpers.size(); j++) {
        dumper = (StatDumper)dumpers.elementAt(j);
        dumper.step();
      }
    }
  }

  public void clear() {
    MetaSink ms;
    StatDumper dumper;
    for (int i=0; i<metaSinks.size(); i++) {
      ms = (MetaSink)metaSinks.elementAt(i);
      Vector dumpers = ms.getStatDumpers();
      for (int j=0; j<dumpers.size(); j++) {
        dumper = (StatDumper)dumpers.elementAt(j);
        dumper.clear();
      }
    }
    metaSinks = new Vector(1,1);
  }
  public void reset() {
    // I don't think it needs to do anything....
  }

  public void finalize() {
    for (int i=0; i<metaSinks.size(); i++) {
      MetaSink ms = (MetaSink)metaSinks.elementAt(i);
      Vector dumpers = ms.getStatDumpers();
      for (int j=0; j<dumpers.size(); j++)
        ((StatDumper)dumpers.elementAt(j)).finalize();
    }
    metaSinks = null;
    super.finalize();
  }
}
