
//Title:        Indeco Steel Simulator
//Version:      
//Copyright:    Copyright (c) 1999
//Author:       Joseph Guse & Ginger Booth
//Company:      Yale FES/CCE
//Description:  Individual-based economic simulator
//of the steel industry, ore->consumer,
//with transportation.

/** StatMinder is a central clearing house for stat-related junk. */


/** Adopted from Indeco for Gecko-in-CourseWare, 990708, vfb. */

package stats;

import java.util.*;
import courseware.util.*;

public class StatMinder {
  private Vector metaPlots = new Vector(2, 1);
  private UI ui; // needs to talk to ui sometimes....
  private boolean amStepping = false;

  public StatMinder(UI ui) {
    this.ui = ui;
  }

  protected UI getUI(){return ui;}
  protected Vector getMetaPlots() {return metaPlots;}

  public void bindParams(ParamList pl) {
    pl.bindAllObjects(this.getClass(), this);
  }

  /** creates new metaPlots vector */
  public void init() {
    metaPlots = new Vector();
    amStepping = false;
  }

  /** clears all MetaPlots--unsuited to running diff scenarios in sequence */
  public void clear() {
    MetaPlot mp;
    for (int i=0; i<metaPlots.size(); i++) {
      mp = (MetaPlot) metaPlots.elementAt(i);
      mp.clear();
    }
    ui.clear();
    amStepping = false;
  }
  /** resets all MetaPlots--traces to time 0 */
  public void reset() {
    MetaPlot mp;
    for (int i=0; i<metaPlots.size(); i++) {
      mp = (MetaPlot) metaPlots.elementAt(i);
      mp.reset();
    }
    amStepping = false;
  }


  public void finalize() {
  }

  public MetaPlot getMetaPlot(String title) {
    for (int i=0; i<metaPlots.size(); i++) {
      MetaPlot mp = (MetaPlot) metaPlots.elementAt(i);
      if (mp.getName().equals(title))
        return mp;
    }
    return null;
  }

  public void addMetaPlot(MetaPlot mp) {metaPlots.addElement(mp);}

  /** The index of the plot title is a valid index for selecting the metaplot
   *  onto a real plot or whatever. The purpose of this thing is to get a list
   * of available plots onto a menu.
   */
  public String[] getPlotTitles() {
    MetaPlot plot;
    int size = metaPlots.size();
    String[] ans = new String[size];
    for (int i=0; i<size; i++) {
      plot = (MetaPlot) metaPlots.elementAt(i);
      ans[i] = plot.getName();
    }
    return ans;
  }

  /** Does nothing if the title doesn't exist. */
  public void showPlot(String title) {
    MetaPlot plot;
    for (int i=0; i<metaPlots.size(); i++) {
      plot = (MetaPlot) metaPlots.elementAt(i);
      if (plot.getName() == title)
        plot.show();
    }
  }

  /** Display/log all metaplots. */
  public void showAll() {
    MetaPlot plot;
    for (int i=0; i<metaPlots.size(); i++) {
      plot = (MetaPlot) metaPlots.elementAt(i);
      plot.show();
    }
  }

  /** has nothing to do but turn the plots on */
  public void run() {
    amStepping = true;
    showAll();
    amStepping = false;
  }

  /** Note that isStepping done doesn't necessarily mean the plots have repainted,
   *  just that the stats have been taken.
   */
  public boolean isStepping() {return amStepping;}

  public void step() {
    amStepping = true;
    int numplots = metaPlots.size();
    for (int i=0; i<numplots; i++) {
      ((MetaPlot)metaPlots.elementAt(i)).step();
    }
    amStepping = false;
  }
}
