
//Title:        Indeco Steel Simulator
//Version:      
//Copyright:    Copyright (c) 1999
//Author:       Joseph Guse & Ginger Booth
//Company:      Yale FES/CCE
//Description:  Individual-based economic simulator
//of the steel industry, ore->consumer,
//with transportation.

/** Adopted from Indeco for Gecko-in-CourseWare, 990708, vfb. */

package stats;

import java.net.URL;
import java.io.*;
import java.util.Vector;

import courseware.util.*;

/** UI = User Interface = the base class of AppletGUI, ApplicationGUI, and
 *  BatchUI.
 *  vfb, 001227, added appendMode for restart support in Gecko.
 */

public abstract class UI {
  private PrintStream errlogMessages;
  private PrintWriter logMessages;
  private String protocolFile;
  static private Vector printWritersOpened;
  static private Vector printWriterFilenames;
  static private boolean appendMode = false;

  /** One possible logStream is System.err.... Send null if no protocolFile. */
  public UI(String protocolFile)
  {
    this.protocolFile = protocolFile;
    errlogMessages = null;
    logMessages = null;
    printWritersOpened = new Vector(1,1);
    printWriterFilenames = new Vector(1,1);
  }
  public void setAppendMode(boolean b) {appendMode = b;}

  abstract public void step();

  /** System.err is a deprecated PrintStream, all others PrintWriter...
   *  You may set both if want system.err AND a file log. */
  public void setErrorLog(PrintStream messageLog) {
    errlogMessages = messageLog;
  }

  /** System.err is a deprecated PrintStream, all others PrintWriter...
   *  You may set both if want system.err AND a file log. */
  public void setLog(PrintWriter messageLog) {
    logMessages = messageLog;
  }

  public void setLog(String filename) {
    logMessages = openFile(filename);
  }

  private PrintWriter findOpen(String filename) {
    for (int i=0; i<printWriterFilenames.size(); i++)
      if ( ( (String) printWriterFilenames.elementAt(i)).equals(filename) )
        return (PrintWriter) printWritersOpened.elementAt(i);
    return null;
  }

  public PrintWriter openFile(String filename) {
    PrintWriter pw = findOpen(filename);
    if (pw != null)
      return pw;
    try {
      pw = new PrintWriter(new FileWriter(filename, appendMode), true);
      printWritersOpened.addElement(pw);
      printWriterFilenames.addElement(filename);
    } catch (Exception e) {
      log("Error : " + e + " Opening output file : " + filename);
    }
    return pw;
  }

  /** Do whatever's necessary to prepare for First Run. */
  public void init() {
    echoInput();
  }

  /** Do whatever's necessary to clean up Between Runs. */
  abstract public void clear() ;
  /** Do whatever's necessary to reset this runs to time 0. */
  abstract public void reset() ;

  public void bindParams(ParamList pl) {
    pl.bindAllObjects(this.getClass(), this);
  }

  /** Hooks statistic source to file or widget or both. */
  public abstract void makeStatDumpers(MetaPlot metaplot);

  /** vector of StatDumpers this Metaplot is on. */
  public abstract Vector getStatDumpers(MetaPlot metaplot);

  public void echoInput() {
    Vector insertedFiles = new Vector(0,1);
    if (protocolFile == null)
      return;
    String readingFile = protocolFile;
    BufferedReader ip = null;
    do {
      try {
        ip = new BufferedReader(new FileReader(readingFile));
        log("// input file : " + readingFile);
        String aline;
        while ((aline = ip.readLine()) != null) {
          log(aline);
          // check for (only) insert files--output them later
          String upper = (new String(aline)).toUpperCase();
          if (upper.indexOf("INSERTFILE") > -1) {
            int quote1 = aline.indexOf('"');
            if (quote1 > 0) {
              int quote2 = aline.indexOf('"', quote1+1);
              if (quote2 > 0) {
                String insfile = aline.substring(quote1+1, quote2);
                insertedFiles.addElement(insfile);
              }
            }
          }
        }
      } catch (Exception e) {
        log("echoInput failed: " + e.getMessage());
      }
      try {
        ip.close();
      } catch (IOException e) {}
      readingFile = null;

      if (insertedFiles.size() > 0) {
        readingFile = (String) insertedFiles.firstElement();
        int lastSlash = protocolFile.lastIndexOf('/');
        if (lastSlash < 0)
          lastSlash = protocolFile.lastIndexOf('\\');
        if (lastSlash > 0)
          readingFile = protocolFile.substring(0,lastSlash+1) + readingFile;
        insertedFiles.removeElementAt(0);
      }
    } while (readingFile != null);
  }

  public void log(String s) {
    if (logMessages != null)
      logMessages.println(s);
    if (errlogMessages != null)
      errlogMessages.println(s);
  }

  public boolean isGUI() { return false;}

  public void finalize() {
    if (logMessages != null) {
      logMessages.flush();
      logMessages.close();
    }
    // don't close System.err....
    if (errlogMessages != System.err) {
      errlogMessages.flush();
      errlogMessages.close();
    }
  }
}